%_____________________________________________________________________________________________________________________
% Parent program to implement the model selection procedure for a generic
% VAR(p) model, as described in the paper 'Model Selection with Judgment'
% Poggiolaia, April 2018
% By Simone Manganelli
%_____________________________________________________________________________________________________________________
clear, rng(1), clc
warning off
parpool
%_____________________________________________________________________________________________________________________
%
% 1. Load inflation data
%_____________________________________________________________________________________________________________________
[DATA, Date] = xlsread('SDWdata_May2018.xlsx', 'A2:F232');
pi = DATA(:,1); % Inflation
piX = DATA(:,2); % Inflation excluding food and energy
un = DATA(:,3); % Unemployment
ip = DATA(:,4); % Industrial production
%Y = [pi,piX,un,ip]';   % Dependent variable
Y = [pi,piX,ip]';   % Dependent variable
%Y = [pi,piX,un]';   % Dependent variable
%Y = [pi,piX]';   % Dependent variable
%Y = [pi]';   % Dependent variable
%_____________________________________________________________________________________________________________________
%
% 2. Initialize a few variables
%_____________________________________________________________________________________________________________________
[g,n]  = size(Y); % Number of dependent variables
g
%-----------------------
% a) Set judgmental parameters
%-----------------------
atilde = 1.9;	% Judgmental decision
htilde = 24;    % Horizon associated with the judgmental decision
alpha  = 0.10;	% Confidence level
%-----------------------
% b) Choose the maximum number of lags
%-----------------------
p      = 12;     % Maximum amount of lags allowed in the VAR(p) model
MR     = round(g*n/50);% Maximum number of regressors (at least 50 observations per regressor)
MM     = g*(1+g*p); % Maximum possible regressors
%_____________________________________________________________________________________________________________________
%
% 3. Start the loop for different horizons
%_____________________________________________________________________________________________________________________
h      = [1,3,6,12,24,36];	 % Months ahead forecast
hn     = length(h);
aBest  = zeros(1,hn); X = cell(1,hn); fval = X; aJudg = X; w = X;
for i = 1:hn
    disp(h(i))
    [aBest(1,i),Xi,fvali,aJudgi,wi] = VARmodelSelection_Patternsearch(Y,atilde,htilde,alpha,p,h(i));
    X(1,i)     = mat2cell(Xi,size(Xi,1),size(Xi,2)); 
    fval(1,i)  = mat2cell(fvali,size(fvali,1),size(fvali,2)); 
    aJudg(1,i) = mat2cell(aJudgi,size(aJudgi,1),size(aJudgi,2)); 
    w(1,i)     = mat2cell(wi,size(wi,1),size(wi,2));
    save('Mod_sel_8_May_2018_VAR3IP_patternsearch_atilde19')
end