function [aBest,X,fval,aJudg,w] = VARmodelSelection_Patternsearch(Y,atilde,htilde,alpha,p,h)
% 
%_____________________________________________________________________________________________________________________
% Compute the best average decision
%_____________________________________________________________________________________________________________________
%
%_____________________________________________________________________________________________________________________
%
% 1. Initialize a few variables
%_____________________________________________________________________________________________________________________
[g,n]  = size(Y); % Number of dependent variables
MR     = round(g*n/50);% Maximum number of regressors (at least 50 observations per regressor)
MM     = g*(1+g*p); % Maximum possible regressors
%_____________________________________________________________________________________________________________________
%
% 2. Start model selection
%_____________________________________________________________________________________________________________________
%-----------------------
% a) Construct the matrix of initial values 
%-----------------------
%STEPS = MR+g^2:2*g^2:g+g^2*p;
STEPS   = [MR+g^2,MR+g*g^2,g+g^2*p];
A       = zeros(length(STEPS)+2,MM);
A(1,:)  = [ones(1,MR),zeros(1,MM-MR)]; % Put in the first MR regressors of the VAR
A(2,:)  = [ones(1,g),zeros(1,MM-g)]; % Put in only the constants
%
% Use also lasso
COUNT = 2;
for j = 1:length(STEPS)
    i = STEPS(j);
    theta1 = VARestimationLasso(Y,1:i,p); % Add also the coefficients selected by lasso 
    idx    = find(sum((theta1~=0)) <= MR);
    if ~isempty(idx)
        theta = theta1(:,idx(1));
        theta(theta1(:,idx(1))~=0)=1;
        if size(theta1,1) < MM
            theta = [theta', zeros(1,MM-length(theta))];
        end
        COUNT      = COUNT+1;
        A(COUNT,:) = theta;
    end
end
A = A(sum(A,2)>0,:);
%-----------------------
% b) Run Patternsearch algorithm
%-----------------------
func    = @(X)VARrisk(X,Y,h,htilde,atilde,alpha,p); % Objective function to be minimized
optionsPattern = optimoptions('patternsearch','Display', 'Iter', 'FunctionTolerance', 1e-10, 'ScaleMesh', 'off', ...
    'MeshTolerance', 1, 'StepTolerance',1, 'SearchFcn', 'MADSPositiveBasisNp1', ...
    'UseCompletePoll',true, 'UseCompleteSearch',true, 'UseParallel',true); %'PollMethod', 'GSSPositiveBasis2N', 
I = size(A,1);   X = zeros(size(A)); fval = zeros(I,1); 
exitflag = fval; aJudg = fval;       err = zeros(n-h,I);
for i = 1:I
    [X(i,:),fval(i),exitflag(i)] = patternsearch(func,A(i,:),ones(1,MM),MR,[],[],zeros(1,MM),ones(1,MM),[],optionsPattern);
    [~,~,err(:,i),aJudg(i)] = VARrisk(X(i,:),Y,h,htilde,atilde,alpha,p);
    save('Mod_sel_8_May_2018_VAR3IP_patternsearch_temp')
end
%-----------------------
% c) Construct the best decision as weighted average
%-----------------------
best = find(fval == min(fval)); best = best(1);% Find the best model
sig = sqrt((0.5*err(:,best).^2-fval(best))'*(0.5*err(:,best).^2-fval(best))/(n-h)); % Standard deviation of the best loss
w = zeros(I,1);
for i = 1:I
    w(i) = 1-normcdf(sqrt(n-h)*(fval(i)-fval(best))/sig); % Weigth of each decision
end
aBest = (w'*aJudg)/sum(w);